@@ -34,6 +34,35 @@ def repairNotes(resNo,chain,missing):
         log.write('The residue No {} in chain {} have missing atom(s) {}.'
         ' We have fixed it'.format(resNo, chain,missing)+'\n\n')
 
+def non_standard_residue(resNo,resn,chain):
+    """
+    The expected standard amino-acid residue names
+    are given in a dictionary at ForcefieldParam.py.
+
+    If user is using PDB file obtained from
+    GROMACS, CHARMM or other MD programs,
+    histidine may be named HSD, HSE or HSP.
+    PRAS will use it as normal HIS because
+    HIS protonation is JUST as described in PRAS paper
+
+    If PRAS encounters a non-standard residue the program
+    will be terminated and user will be informed
+
+    Arguments
+    ----------
+    resNo:   the amino acid residue number
+    resn:    the amino acid residue name
+    chain:   the chain or chain number
+
+    Returns
+    -------
+    None:    a premature termination of the program
+    """
+    with open('non_standard_residue.txt', 'w') as log:
+        log.write('The residue {}  named {} in chain {} is a non-standard residue.'
+        ' Program has terminated abnormally'.format(resNo,resn,chain)+'\n\n')
+        sys.exit()
+
 def exitNotes(resNo,chain):
     """
     PRAS can fix missing backbone O but demands that N, CA, & C be present.
@@ -109,7 +138,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
         ' Add Missing Heavy Atoms and Hydrogen Atoms and Assign'+'\n'
         'Secondary Structure by Amide Interactions. Submitted'+'\n')
         log.write('#'*118+'\n')
-        log.write('PRAS 1.0.5 This is a PRAS-generated log file.'
+        log.write('PRAS 1.0.6 This is a PRAS-generated log file.'
                 ' For your information, fixed atoms (if any) are appended below '+'\n')
         log.write('#'*118+'\n\n')
 
@@ -128,13 +157,13 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
     """read user's input file to be repaired or analyzed
         but if it is not readable inform the user and
         terminate the program"""
-    #try:
-    chains = getChains(pdb_pras,rotamer,mutation,"")
-    #except:
-        #chains = []
+    try:
+        chains = getChains(pdb_pras,rotamer,mutation,"")
+    except:
+        chains = []
 
-    #if not chains:
-    #    exitError()
+    if not chains:
+        exitError()
 
     for n in range(len(chains)):
         resn,atoms,atmpos,resNo,chain_id = chains[n][0],\
@@ -144,10 +173,19 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
         # first, check if cterminal is missing a backbone atom and terminate program
         if not  ('N' in atoms[-1] or 'CA' in atoms[-1] or 'C' in atoms[-1]):
             exitNotes(resNo[-1],n+1)
+
         # now loop through all the chains, checking and fixing missing heavy  atoms
         for k in range(len(atoms)):
-            # the expected complete atoms of any residue
-            x = list(param[resseq[k]].keys())[:-1]
+
+            try:
+                x = list(param[resseq[k]].keys())[:-1]
+            except:
+                if any(resseq[k].startswith(i) for i in ['HSD','HSP','HSE']):
+                    resseq[k] = 'HIS' # replace with HIS
+                    x = list(param[resseq[k]].keys())[:-1]
+                else:
+                    non_standard_residue(resNo[k],resseq[k],n+1) # non-standard residues not allowed
+
 
             if resseq[k] == 'ASP':
                 missing=[i for i in x if not i in atoms[k]]
@@ -160,6 +198,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'ASN':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -171,6 +210,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'ALA':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -182,6 +222,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'ARG':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -193,6 +234,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'LEU':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -204,17 +246,25 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'ILE':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
                     pass
                 elif missing and not ('N' in missing or 'CA' in missing or 'C' in missing):
-                    psi = [atmpos[k-1][0],atmpos[k-1][1],atmpos[k-1][2],atmpos[k][0]]
-                    atoms[k],atmpos[k]=repairIle(atoms[k],atmpos[k],resNo[k],n,missing,faspr,atmpos[(k+1)%len(atmpos)],psi)
-                    repairNotes(resNo[k], n+1,missing)
+                    if 'CD' in missing: # CD1 is named CD by GROMACS, CHARMM, etc.
+                        missing[missing.index('CD')] = 'CD1'
+                    if len(missing) == 1:
+                        atoms[k][atoms[k].index('CD')] = 'CD1'
+                        pass
+                    else:
+                        psi = [atmpos[k-1][0],atmpos[k-1][1],atmpos[k-1][2],atmpos[k][0]]
+                        atoms[k],atmpos[k]=repairIle(atoms[k],atmpos[k],resNo[k],n,missing,faspr,atmpos[(k+1)%len(atmpos)],psi)
+                        repairNotes(resNo[k], n+1,missing)
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'PRO':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -226,6 +276,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'SER':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -237,6 +288,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'VAL':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -248,6 +300,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'LYS':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -259,6 +312,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'TYR':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -270,6 +324,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'TRP':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -281,6 +336,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'MET':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -292,6 +348,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'HIS':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -303,6 +360,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'GLY':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -314,6 +372,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'CYS':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -325,6 +384,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'GLU':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -336,6 +396,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'PHE':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -347,6 +408,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'GLN':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -358,6 +420,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                 elif ('N' in missing or 'CA' in missing or 'C' in missing):
                     exitNotes(resNo[k],n+1)
 
+
             elif resseq[k] == 'THR':
                 missing=[i for i in x if not i in atoms[k]]
                 if not missing:
@@ -370,7 +433,7 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                     exitNotes(resNo[k],n+1)
 
         # add C-terminal O [OXT] if it is missing
-        if not 'OXT' in atoms[-1] or 'OC2' in atoms[-1]:
+        if not 'OXT' in atoms[-1]:
             atmpos[-1].extend(c_termini(atoms[-1],atmpos[-1]))
             atoms[-1].extend(['OXT'])
             repairNotes(resNo[-1], n+1,['OXT'])
@@ -384,4 +447,4 @@ def checkpdbAtoms (pdb_pras,rotamer,mutation,pdb_faspr):
                     sg_coord.append(atom_pos[i][1][j])
 
         ter_all.append([resn,atoms,atmpos,resNo,resseq,sg_coord,chain_id])
-    return  ter_all
\ No newline at end of file
+    return  ter_all
