@@ -113,8 +113,7 @@ def fixheavyAtoms(pdb_pras,rotamer,mutation,pdb_faspr):
 
     nchain = checkpdbAtoms(pdb_pras,rotamer,mutation,pdb_faspr)
     chains = list(itertools.chain(*[i[-1] for i in nchain]))
-    if len(chains) != len(nchain):#PDB with unnamed chains
-        chains = list(string.ascii_uppercase[0:len(nchain)])
+    
     number = 1
     for n in range(len(nchain)):
         resn,atom,atmpos,resNo,resseq,sg_coord,chain = nchain[n]
@@ -147,4 +146,4 @@ def fixheavyAtoms(pdb_pras,rotamer,mutation,pdb_faspr):
                     ('TER',' ',number,' ',' ', ' ',resseq[len(x)-1],
                     ' '+chains[n],resNo[len(x)-1]))
                 f.write('END')
-                f.close()
\ No newline at end of file
+                f.close()
