@@ -35,7 +35,7 @@ def calcCoordinate(a,b,c,bond_len,di_angle,theta):
 
     u = c - b
     x = a - b
-    v = ((a - b) - (np.dot((a - b), u)/np.dot(u,u))*u)
+    v = ((x) - (np.dot((x), u)/np.dot(u,u))*u)
 
     w = np.cross(u, x)
 
@@ -94,7 +94,7 @@ def calcTorsionAngle(coord1, coord2, coord3, coord4):
 
 def rotMatrix(axis,theta):
     """
-    This functinn generates 3D rotation matrix
+    This function generates 3D rotation matrix
     for a rotation about an axis.
 
     Arguments
@@ -130,7 +130,7 @@ def rotMatrix(axis,theta):
 
 def arbRot(vector,axis,theta):
     """
-    This functinn generates a new position vector after
+    This function generates a new position vector after
     rotating a vector by an angle through an axis.
 
     Arguments
