@@ -102,7 +102,7 @@ def repairPDB(pdb_pras,rotamer,mutation,pdb_faspr):
                right conformation for the amino acid residue side-chain. It is adviced that
                you compare both methods when necessary.
 
-               Be mindful of the fact that FASPR may be is less flexible with reading
+               Be mindful of the fact that FASPR may be less flexible with reading
                PDB files and has no mechanism to use lower conformers in the case
                of rotamers/mutation. To avoid manual editing, pass the PDB through PRAS first.
 
@@ -188,7 +188,7 @@ def repairPDB(pdb_pras,rotamer,mutation,pdb_faspr):
                     apos,name=glycine_h(x[i])
                     res_pos.extend([apos]);atom_name.extend([name])
 
-            elif j[0].startswith('HIS'):
+            elif j[0].startswith('HIS') or any(j[0].startswith(hs) for hs in ['HSD','HSP','HSE']):
                 if x[(i+1)% len(x)][0][:3] != 'PRO' and x[i] != x[-1]:
                     apos,hpos,name=histidine_h(x[i],i,x[(i+1)%len(x)])
                     res_pos.extend([apos]);h_pos.extend([hpos]);atom_name.extend([name])
@@ -312,7 +312,7 @@ def repairPDB(pdb_pras,rotamer,mutation,pdb_faspr):
                         res_pos.extend([apos])
                         atom_name.extend([name])
 
-        ##################################################################################################################
+        # Histidine protonation
         # comment next code line if you want all HIS side-chain neutral. See explanation online at www.protein-science.com
         # this code line randomly protonates 20% of all HIS residues
         res_pos, atom_name = prot_his(x,res_pos,atom_name,n)
