@@ -84,7 +84,7 @@ def calcCoordinate(a,b,c,bond_len,di_angle,theta):
 
     u = c - b
     x = a - b
-    v = ((a - b) - (np.dot((a - b), u)/np.dot(u,u))*u)
+    v = ((x) - (np.dot((x), u)/np.dot(u,u))*u)
 
     w = np.cross(u, x)
 
@@ -199,4 +199,4 @@ def optmizeH(num,atom,atmpos,resNo,resn,data):
         min_H_pos = opt_h[min_energy]
         return  min_H_pos
     except:
-        return data[3]
\ No newline at end of file
+        return data[3]
